# Puppet package\_alternatives module

This module provides a defined type `package_alternatives` for easy
installation of software packages with different (alternative)
names across platforms.

### Requirements

Module has been tested on:

* Puppet 5.5
* Debian and Red Hat family systems

Required modules:

* [puppetlabs-stdlib](https://github.com/puppetlabs/puppetlabs-stdlib)

# Quick Start

Full configuration options:

```
package_alternatives { 'resource title':
  ensure       => ...,         # Resource package ensure state
  fail_missing => boolean,     # Fail on no alternative package.
  alternatives => string|hash, # Structure with package alternatives.
  platform     => string,      # Current platform detection override.
}
```

### Software alternatives structure

`$alternatives` accepts:

* string for no alternative (defaults to resource `$title`)

* hash of platforms and alternative package names, e.g.:

```puppet
$software = {
  'debian' => 'openssh-client',
  'redhat' => 'openssh-clients'
}
```

* hash of resource names with with hash or string containing
  package name or platforms hash of alternatives, e.g.:

```puppet
$software = {
  'bash' => 'bash',
  'openssh-client' => {
    'debian' => 'openssh-client',
    'redhat' => 'openssh-clients',
  }
}
```


### Platforms resolution order

Lower cased:

0. Custom `$platform` parameter
1. `${::operatingsystem}-${::operatingsystemrelease}-${::architecture}` (*centos-6.4-x86_64*)
2. `${::operatingsystem}-${::operatingsystemrelease}` (*centos-6.4*)
3. `${::operatingsystem}-${::operatingsystemmajrelease}-${::architecture}` (*centos-6-x86_64*)
4. `${::operatingsystem}-${::operatingsystemmajrelease}` (*centos-6*)
5. `${::operatingsystem}-${::architecture}` (*centos-x86_64*)
6. `${::operatingsystem}` (*centos*)
7. `${::osfamily}-${::operatingsystemrelease}-${::architecture}` (*redhat-6.4-x86_64*)
8. `${::osfamily}-${::operatingsystemrelease}` (*redhat-6.4*)
9. `${::osfamily}-${::operatingsystemmajrelease}-${::architecture}` (*redhat-6-x86_64*)
10. `${::osfamily}-${::operatingsystemmajrelease}` (*redhat-6*)
11. `${::osfamily}-${::architecture}` (*redhat-x86_64*)
12. `${::osfamily}` (*redhat*)

### Software alternatives structure

Parameter `$alternatives` accepts:

* string for no alternative (defaults to resource `$title`)

```puppet
$software = 'bash'
```

* hash of platforms and alternative package names, e.g.:

```puppet
$software = {
  'debian' => 'openssh-client',
  'redhat' => 'openssh-clients'
}
```

* hash of resource names with with hash or string containing
  package name or platforms hash of alternatives, e.g.:


```puppet
$software = {
  'bash' => 'bash',
  'openssh-client' => {
    'debian' => 'openssh-client',
    'redhat' => 'openssh-clients',
  }
}
```

# Example

```puppet
$software = {
  'rsync'  => 'rsync',          # same name on all platforms
  'ash'    => {
    'debian'   => 'ash'         # just Debian supported
  },
  'tshark' => {
    'debian-6' => 'tshark',     # just Debian 6 and any
    'redhat'   => 'wireshark',  # RedHat family supported
  },
}

package_alternatives { keys($software):
  ensure       => present,
  alternatives => $software,
  fail_missing => false,       # skip packages with no alternative on platform
}
```

***

CERIT Scientific Cloud, <support@cerit-sc.cz>
